#include <sys/types.h>
#include <sys/time.h>
#include <sys/uio.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <signal.h>
#include <stdarg.h>
#include <stdio.h>
#include <syslog.h>
#include <errno.h>
#include "msgstore.h"
#include "spamtest.h"

#define MYDOMAIN        "example.com"
#define TIMEOUT_MS      30000 /* we'll use it per sts_* call */
#define MAX_MSGBUF      1024000

#define SPAM            1
#define PROBABLE_SPAM   2
#define NOT_SPAM        3

#define FILTER_ERROR    4
#define SENDER_ERROR    5
#define CODE_ERROR      10

typedef struct {
        char    *ip;
        char    *helo;
        char    *from;
        char    *rcpt;
        char    *server;
        char    *filename;
} message_info_t;

message_info_t info;

int verbose = 0;

const int max_message_rss = 1024 * 1024 * 10;

int send_body_and_process_result(FILE *f, spamtest_session_t *s, 
    long long *bytes);

void
warning(const char *fmt, ...)
{
        va_list ap;
        char buff[512];

        va_start(ap, fmt);
        vsnprintf(buff, sizeof(buff), fmt, ap);
        syslog(LOG_ERR | LOG_LOCAL1, "%s", buff);
        va_end(ap);
}

void
free_info(message_info_t *info)
{
        free(info->ip);
        free(info->helo);
        free(info->from);
        free(info->rcpt);
        free(info->server);
        free(info->filename);
}

int
process(FILE *f, message_info_t *info, long long *bytes)
{
        spamtest_session_t s;
        int r;
        char *comma, *p;

        r = 0;
        memset(&s, 0, sizeof(s));
        s.access_address = info->server;
        s.mail_domain = MYDOMAIN;
        s.rw_timeout= TIMEOUT_MS;
        s.connect_timeout= TIMEOUT_MS;
        s.flags = STS_USE_POLL;
        
        r = sts_init(&s);
        if (r != 0) {
                warning("sts_init: %s", sts_strerror(r));
                return FILTER_ERROR;
        }

        r = sts_prepare_envelope(&s, info->from, NULL, info->ip, NULL);
        if (r != 0) {
                warning("sts_prepare_envelope: %s", sts_strerror(r));
                return FILTER_ERROR;
        }
        
        comma = p = info->rcpt;
        while (comma != NULL) {
                comma = strchr(p, ',');
                if (comma != NULL) {
                        *comma = 0x0;
                        r = sts_prepare_envelope(&s, NULL, p, NULL, NULL);
                        if (r != 0) {
                                warning("sts_prepare_envelope: %s", 
                                    sts_strerror(r));
                                return FILTER_ERROR;
                        }
                        comma++;
                        p = comma;
                } else {
                        r = sts_prepare_envelope(&s, NULL, p, NULL, NULL);
                        if (r != 0) {
                                warning("sts_prepare_envelope: %s", 
                                    sts_strerror(r));
                                return FILTER_ERROR;
                        }
                }
        }

        r = sts_add_attribute(&s, "helo_name", info->helo);
        if (r != 0) {
                warning("sts_add_attribute: %s", sts_strerror(r));
                return FILTER_ERROR;
        }

        r = send_body_and_process_result(f, &s, bytes);
        sts_close(&s);

        return r;
}

int
headers_check(message_status_t *mst)
{
        int i;
        hdraction_t *header_action;
        int r = 0;

        if (mst->action == STS_ACTION_CHANGE) {
                for (i = 0; i < mst->action_count; i++) {
                        header_action = &mst->act_array[i];
                        switch (header_action->type) {
                        case STS_HEADER_ADD:
                        case STS_HEADER_NEW:
                        case STS_HEADER_CHG:
                                if (verbose) {
                                        fprintf(stdout, "%s: %s\n", header_action->header, header_action->value);
                                }
                                if (!strcmp(header_action->header, 
                                        "X-SpamTest-Status")) {
                                        if (!strcmp(header_action->value, 
                                                "SPAM")) {
                                                r = SPAM;
                                        }
                                        if (!strcmp(header_action->value, 
                                                "Probable Spam")) {
                                                r = PROBABLE_SPAM;
                                        }
                                }
                                break;
                        default:;
                        }
                }
        }
        
        if (r) return r;
        else return NOT_SPAM;
}

int
send_body_and_process_result(FILE *f, spamtest_session_t *s, long long *bytes)
{
        char linebuf[2048];
        int l;
        spamtest_status_t *st = NULL;
        message_status_t *mst;
        int status = 0;

        /* message processing loop */
        while (fgets(linebuf, sizeof(linebuf) - 1, f)) {
	        l = strlen(linebuf);
	        *bytes += l;

                /* simply send linebuf to filter */
                if (!(st = sts_send_body(s, linebuf, l, 0))) {
                        status = FILTER_ERROR;
                        warning("Filtering error: sts_send_body");
                        goto end;
                }
             
                /* analyze result code */
                switch(st->status) {
                default:
		    warning("Unknown result code: %d", st->status);
		    /* no break here */
                case STS_SENDERROR:
                    status = SENDER_ERROR;
                    goto end;
                case STS_CONTINUE:
		    /* do nothing */
                    break;
                case STS_ACCEPT:
                case STS_FILTER_ERROR:
                case STS_REJECT:
                case STS_BLACKHOLE:
                case STS_SMTP_ACCEPT:
                    goto ready;
                }
        }

        st = sts_body_end(s, 0);
        if (!st) {
                status = FILTER_ERROR;
                goto end;
        }
        
ready:
        /* now we have message status in st, message body in message
         * store, let's go*/
        switch (st->status) {
        case STS_ACCEPT:
                st = glue_actions(st);
                mst = &st->ms_array[0];
                status = headers_check(mst);
	        break;
	case STS_SMTP_ACCEPT:
                status = NOT_SPAM;
	        break;
	case STS_BLACKHOLE:
                status = SPAM;
                break;
	case STS_FILTER_ERROR:
                warning("filter error");
                status = FILTER_ERROR;
                break;
	case STS_SENDERROR:
                warning("sender error");
                status = SENDER_ERROR;
                break;
	case STS_REJECT:
                status = SPAM;
                break;
	default:
                status = CODE_ERROR;
                break;
	}
        
end:
        return status;
}

void
usage(char *p)
{
        fprintf(stderr, "Usage: %s [params] spamtest-server inputfile\n", p);
        fprintf(stderr, "Param:\n"
                "\t-i <ip>      -- ip address of relay\n"
                "\t-e <str>     -- ehlo (helo) given by last relay\n"
                "\t-f <str>     -- env. from given by last relay\n"
                "\t-r <str>     -- comma separated list of recipients\n"
                "\t-h           -- this help\n");
        exit(0);
}

int
main(int argc, char **argv)
{
        int c, use_file, rc;
        char *progname;
        long long int bytes;
        FILE *f;

        use_file = 0;
        f = NULL;
        bytes = 0;
        verbose = 0;
        memset(&info, 0, sizeof(info));

        progname = strdup(argv[0]);
        
        while ((c = getopt(argc, argv, "i:f:r:e:hv")) != -1) {
                switch (c) {
                case 'h':
                        usage(progname);
                        break;
                case 'i':
                        info.ip = strdup(optarg);
                        break;
                case 'f':
                        info.from = strdup(optarg);
                        break;
                case 'r':
                        info.rcpt = strdup(optarg);
                        break;
                case 'e':
                        info.helo = strdup(optarg);
                        break;
                case 'v':
                        verbose = 1;
                        break;
                default:
                        usage(progname);
                }
        }

        argc -= optind;
        argv += optind;

        info.server = argv[0] ? strdup(argv[0]) : NULL;
        info.filename = argv[1] ? strdup(argv[1]) : NULL;

        if (!info.server) {
                usage(progname);
        }

        if (info.filename) {
                f = fopen(info.filename, "r");
                if (!f) {
                        warning("can't open file %s: %s",
                            info.filename, strerror(errno));
                        exit(CODE_ERROR);
                } else {
                        use_file = 1;
                }
        } else {
                f = stdin;
        }

        signal(SIGPIPE,SIG_IGN);
        openlog("spamtest-checker", LOG_PID, LOG_LOCAL1);

        rc = process(f, &info, &bytes);

        if (verbose) {
                if (rc == SPAM) {
                        fprintf(stderr, "SPAM\n");
                } else if (rc == PROBABLE_SPAM) {
                        fprintf(stderr, "PROBABLE SPAM\n");
                } else if (rc == NOT_SPAM) {
                        fprintf(stderr, "NOT SPAM\n");
                } else {
                        fprintf(stderr, "filter error, see logs\n");
                }
        }
        
        if (use_file) {
                fclose(f);
        }
        
        free(progname);
        free_info(&info);
        
        return rc;
}
