/* -*- C -*-
 * Spamtest connection library
 * File: bufio.c
 *      (C)  Alex Tutubalin <lexa@lexa.ru>
 *	Apr-May 2003
 *
 * Created: Wed Apr 23 22:08:08 2003
 */

#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <poll.h>
#include <sys/types.h>
#include <sys/time.h>
#include <string.h>
#include <errno.h>
#include <syslog.h>
#include <stdio.h>

#include "bufio.h"
#include "heap.h"

int calc_timeout(spamtest_session_t *s, int get_timediff);

#if (defined (_THREAD_SAFE) || defined (_REENTRANT)) && defined __USE_LOCKING
#include <pthread.h>
static pthread_mutex_t bufio_mutex;

void local_bufio_library_init(void)
{
    pthread_mutex_init(&bufio_mutex,NULL);
}

static void __lock(void)
{
    pthread_mutex_lock(&bufio_mutex);
}
static void __unlock(void)
{
    pthread_mutex_unlock(&bufio_mutex);
}
#else
static void __unlock(void){}
static void __lock(void){}
void local_bufio_library_init(void){}

#endif

void bufio_setflags(bufio_t* b, unsigned int flags)
{
    if(b)
	b->flags=flags;
}
unsigned int bufio_getflags(bufio_t *b)
{
    return b?b->flags:0;
}

bufio_t* bufio_init(msg_heap_t *heap, int file, int rbufs, int wbufs)
{
    bufio_t *ret;
    int flags;

    if(file<0) return NULL;

    __lock();
    ret = heap_allocz(heap,sizeof(bufio_t));
    if(!ret) 
	{
	    __unlock();
	    return NULL;
	}
    
    ret->fd = file;
    flags = fcntl(ret->fd, F_GETFL, 0);
    if(fcntl(ret->fd, F_SETFL, flags | O_NONBLOCK) == -1)
	{
	    __unlock();
	    return NULL;
	}

    /* alloc buffers */
    ret->rbuf.buf = heap_alloc(heap,ret->rbuf.alloc = rbufs>0?rbufs:DEFAULT_BUFSIZE);
    if(!ret->rbuf.buf)
	{
	    __unlock();
	    return NULL;
	}
    ret->wbuf.buf = heap_alloc(heap,ret->wbuf.alloc = wbufs>0?wbufs:DEFAULT_BUFSIZE);
    if(!ret->wbuf.buf)
	{
	    __unlock();
	    return NULL;
	}
    __unlock();
    return ret;
}

int bufio_read_some_data(bufio_t *buf, spamtest_session_t *s)
{
    fd_set rfd;
    struct pollfd pfd;
    int can_read,rbytes,to_read;
    struct timeval tv;
    if(!buf)
	return BUFIO_ERR;
    if(s->timeout_nq<=0) 
	if(calc_timeout(s,1)!=STS_ERR_NO_ERR)
	    return BUFIO_TIMEOUT;
    while(1)
	{
	    if(buf->flags & BUFIO_USE_POLL)
		{
		    pfd.fd = buf->fd;
#ifdef POLLDRNORM
		    pfd.events =(POLLIN|POLLRDNORM|POLLRDBAND);
#else
		    pfd.events =(POLLIN);
#endif
		    pfd.revents = 0;
		    can_read=poll(&pfd,1,s->timeout_curr);
		    if(can_read >0)
			{
#ifdef POLLDRNORM
			    can_read = =pfd.revents & (POLLIN|POLLRDNORM|POLLRDBAND);
#else
			    can_read = pfd.revents & POLLIN;
#endif
			}
		}
	    else
		{
		    tv.tv_sec = s->timeout_curr/1000;
		    tv.tv_usec = (s->timeout_curr%1000)*1000;
		    FD_ZERO(&rfd);
		    FD_SET(buf->fd,&rfd);
		    can_read=select(buf->fd+1,&rfd,NULL,NULL,&tv);
		}
	    if(can_read <0)
		{
		    if(errno == EINTR)
			continue;
		    else
			{
			    buf->rbuf.pending_error = errno;
			    return BUFIO_ERR;
			}
		}
	    if(can_read==0) 
		{
		    if(--s->timeout_nq <= 0)
			if(calc_timeout(s,1)!=STS_ERR_NO_ERR)
			    return BUFIO_TIMEOUT;
		    continue;
		}
	    if(buf->flags & BUFIO_USE_POLL)
		{
		    // nothing, can_read already set
		}
	    else
		{
		    if(!FD_ISSET(buf->fd,&rfd)) 
			{
			    return BUFIO_WAITDATA;
			}
		}
	    to_read = buf->rbuf.alloc - buf->rbuf.lastbyte;
	    rbytes=read(buf->fd, buf->rbuf.buf+buf->rbuf.lastbyte,to_read);
	    if(rbytes <0)
		{
		    switch(errno)
			{
			case EWOULDBLOCK:
			case EINTR:
			    return BUFIO_WAITDATA;
			default:
			    buf->rbuf.pending_error = errno;
			    return BUFIO_ERR;
			}
		}
	    else if(rbytes == 0)
		{
		    return BUFIO_EOF;
		}
	    else
		{
		    buf->rbuf.lastbyte+=rbytes;
		    return BUFIO_OK;
		}
	}
}

/* get 1 byte from input stream */
int bufio_getbyte(bufio_t *buf,unsigned char *byte, spamtest_session_t *s)
{
    int ret;
    __lock();
    if(buf->rbuf.firstbyte<buf->rbuf.lastbyte)
	{
	    *byte=buf->rbuf.buf[buf->rbuf.firstbyte++];
	    __unlock();
	    return BUFIO_OK;
	}
    /* we've read entire buffer */
    buf->rbuf.firstbyte = buf->rbuf.lastbyte = 0;
    while(BUFIO_WAITDATA==(ret=bufio_read_some_data(buf,s)))
	;
    if(ret == BUFIO_OK)
	{
	    if(buf->rbuf.firstbyte<buf->rbuf.lastbyte)
		{
		    *byte=buf->rbuf.buf[buf->rbuf.firstbyte++];
		    __unlock();
		    return BUFIO_OK;
		}
	    else
		{
		    __unlock();
		    return BUFIO_EOF;
		}
	}
    else
	{
	    __unlock();
	    return ret;
	}
}

int bufio_read(bufio_t *buf, void *ptr, int size,spamtest_session_t *s)
{
    unsigned char *retbuf = (unsigned char *)ptr;
    int ret,count;
    __lock();
    if((ret = buf->rbuf.pending_error))
	{
	    buf->rbuf.pending_error = 0;
	    __unlock();
	    return -1;
	}
    count = 0;
    __unlock();
    while(count<size)
	{
	    ret = bufio_getbyte(buf,&retbuf[count],s);
	    __lock();
	    if(ret!=BUFIO_OK)
		{
		    buf->rbuf.pending_error = ret;
		    __unlock();
		    return count;
		}
	    else
		count++;
	    __unlock();
	}
    return count;
}


int bufio_flush(bufio_t *buf,spamtest_session_t *s)
{
    int written;
    fd_set wfd;
    struct pollfd pfd;
    int can_write;
    struct timeval tv;

    __lock();
    if(s->timeout_nq<=0) 
	if(calc_timeout(s,1)!=STS_ERR_NO_ERR)
	    {
		__unlock();
		return BUFIO_TIMEOUT;
	    }

    while(buf->wbuf.firstbyte < buf->wbuf.lastbyte)
	{
	    if(buf->flags & BUFIO_USE_POLL)
		{
		    pfd.fd = buf->fd;
#ifdef POLLDRNORM
		    pfd.events =(POLLOUT|POLLWRNORM|POLLWRBAND);
#else
		    pfd.events =(POLLOUT);
#endif
		    pfd.revents = 0;
		    can_write=poll(&pfd,1,s->timeout_curr);
		    if(can_write >0)
			{
#ifdef POLLDRNORM
			    can_write = =pfd.revents & (POLLOUT|POLLWRNORM|POLLWRBAND);
#else
			can_write = pfd.revents & POLLOUT;
#endif
			}
		}
	    else
		{
		    tv.tv_sec = s->timeout_curr/1000;
		    tv.tv_usec = (s->timeout_curr%1000)*1000;
	    	    FD_ZERO(&wfd);
		    FD_SET(buf->fd,&wfd);
		    can_write=select(buf->fd+1,NULL,&wfd,NULL,&tv);
		}
	    if(can_write <0)
		{
		    if(errno == EINTR)
			continue;
		    else
			{
			    __unlock();
			    return BUFIO_ERR;
			}
		}
	    if(can_write==0) 
		{
		    if(--s->timeout_nq <= 0)
			if(calc_timeout(s,1)!=STS_ERR_NO_ERR)
			    {
				__unlock();
				return BUFIO_TIMEOUT;
			    }
		    continue;
		}
	    if(buf->flags & BUFIO_USE_POLL)
		{
		    // nothing, can_write already set
		}
	    else
		{
		    if(!FD_ISSET(buf->fd,&wfd)) 
			{
			    __unlock();
			    return BUFIO_CANNOTWRT;
			}
		}
	    written = write(buf->fd,buf->wbuf.buf+buf->wbuf.firstbyte,
			    buf->wbuf.lastbyte-buf->wbuf.firstbyte);
	    if(written<0)
		{
		    if(errno != EWOULDBLOCK)
			{
			    __unlock();
			    return BUFIO_ERR;
			}
		    else
			continue;
		}
	    else if(written == 0)
		{
		    continue;
		}
	    else
		buf->wbuf.firstbyte +=written;

	}
    buf->wbuf.firstbyte = buf->wbuf.lastbyte = 0;
    __unlock();
    return BUFIO_OK; /* nothing to flush */
}


int bufio_putbyte(bufio_t *buf, unsigned char byte,spamtest_session_t *s)
{
    int ret;
    while(buf->wbuf.lastbyte == buf->wbuf.alloc)
	if((ret=bufio_flush(buf,s))!=BUFIO_OK)
	    return ret;
    __lock();
    buf->wbuf.buf[buf->wbuf.lastbyte++] = byte;
    __unlock();
    return BUFIO_OK;
}

int bufio_write(bufio_t *buf, const void *data, int maxsize,spamtest_session_t *s)
{
    int count;
    int ret;
    unsigned char *dptr;
    __lock();
    count=0;
    dptr = (unsigned char *)data;
    if(buf->wbuf.pending_error)
	{
	    ret = buf->wbuf.pending_error;
	    buf->wbuf.pending_error=0;
	    __unlock();
	    return ret;
	}
    __unlock();
    while(count<maxsize)
	{
	    if(BUFIO_OK!=(ret=bufio_putbyte(buf,dptr[count],s)))
		{
		    buf->wbuf.pending_error = ret;
		    return count>0?count:-1;
		}
	    count++;
	}
    return count;
}

