/* -*- C -*-
 * File: kas_defs.h
 *
 * Created: Sat Oct  2 09:46:09 2004
 */

#ifndef __KAS_DEFS_H
#define __KAS_DEFS_H

#include <sys/time.h>
#include <sys/types.h>

#undef EXTERN_C
#undef EXTERN_C_BEGIN
#undef EXTERN_C_END

#ifdef __cplusplus
#  define       EXTERN_C                extern "C"
#  define       EXTERN_C_BEGIN          extern "C" {
#  define       EXTERN_C_END            }
#else
#  define       EXTERN_C                extern
#  define       EXTERN_C_BEGIN
#  define       EXTERN_C_END
#endif /* ifdef __cplusplus */

EXTERN_C_BEGIN


/* error codes */
enum kas_error_values 
{
    KASerrOK=0,
    KASerrPARAMS_NULLPTR=-1,	// NULL pointer passed instead of  kas_session_t *
    KASerrPARAMS_NOTINITED=-2,	// Not initialized session data passed    
    KASerrALREADY_CONNECTED=-3,	// session already connected (in connect)
    KASerrUNABLE_TO_CONNECT=-4,	// connect returns -1
    KASerrIN_PROGRESS=-5,	// connect in progress
    KASerrREAD_ERROR=-6,	// error while read
    KASerrWRITE_ERROR=-7,	// error while read
    KASerrDISCONNECTED=-8,	// attempt to IO on disconnected  session
    KASerrFILEERROR=-9,	// Cannot open file
    KASerrBUSY=-10,		// Ping/Scan for not-idle session
    KASerrINDEX_RANGE=-11,	// multisession index out of range
    KASerrIO_ERROR=-12,		// cannot distinguish between
				// read/writer error in multisession
};


enum kas_status_value 
{
    // true states - values for kas_status_t.status
    KAS_STATUS_DISCONNECTED	=1, // Session disconnected
    KAS_STATUS_CONNECTING	=2, // connect() called, but connect state not reached
    KAS_STATUS_CONNECTED	=4, // connected
    KAS_STATUS_IDLE		=8, // IDLE
    KAS_STATUS_PINGWRITE	=16, // sending ping data 
    KAS_STATUS_PINGRESPONSE	=32, // receiving Ping-reply
    KAS_STATUS_SENDING		=64, // message data writing
    KAS_STATUS_RECEIVING	= 128, // receiving status
    KAS_STATUS_WRIDLE		= 256, // Idle after data portion
    
    // these status values are used in information progress callback calls
    // in new_status field
    KAS_STATUS_CLOSING		=(1<<11), // called in KasDisconnect
    KAS_STATUS_SCAN_END		=(1<<12), // called after message status processing
    KAS_STATUS_WANTRESTART	=(1<<13), // filtering engine want to restart

    // these 3 information callbacks called before Disconnect due to error
    KAS_STATUS_IOERROR		=(1<<14), // read/write error
    KAS_STATUS_CONNECTERROR	=(1<<15), // connect error
    KAS_STATUS_INCORRECT_PROTO  =(1<<16)  // Incorect protocol sequence
};

// Extended status for IOERROR/CONNECTERROR information callback
enum kas_extdstatus_value 
{
    KAS_ES_NOERROR=0, // no error (for all other callbacks)
    KAS_ES_CONNERROR=1,   // connect() returns an error
    KAS_ES_CONNTIMEOUT=2, // connect timeout
    KAS_ES_IOERROR=3, // read/write error
    KAS_ES_IOTIMEOUT=4 // read/write timeout
};

typedef struct 
{
    enum kas_status_value	status;      // KAS_STATUS_*
    enum kas_extdstatus_value	extd_status; // nonzero for
					     // KAS_STATUS_*ERROR
					     // values
    struct timeval		elapsed_time; // time elapsed for
					      // ScanFile/ScanOpenedFile operation
    pid_t			scaner_pid;  // 
					     // scanner pid (not known for DISCONNECTED, 
					     //CONNECTING and CONNECTED statuses, known for IDLE)
} kas_status_t;


// Status callback prototype
typedef struct kas_session_s       kas_session_t; // forward decl
typedef struct kas_multisession_s       kas_multisession_t; // forward decl
typedef void (kas_cb_single_status_t)(kas_session_t* _session,
                                     kas_status_t *old_status,
                                     kas_status_t *new_status,
                                     void* param);

typedef void (kas_cb_multi_status_t)(kas_multisession_t* _sessionlist,
				     unsigned int index,
                                     kas_status_t *old_status,
                                     kas_status_t *new_status,
                                     void* param);


// Progress callback

enum kas_progress_cb_value {
  KAS_INFO_STARTSEND = 1, // star passing message data to filter
  KAS_INFO_WAITSTATUS=2,  // waiting for message status
  KAS_INFO_DONE=3,        // callback with message status (all other fields in  
			  // kas_progress_info_t is also set
  KAS_INFO_ERROR=4,       // I/O or protocol error, called before
			  // Disconnect
  KAS_INFO_CONTSEND=5,    // continue send in chunked mode
};
enum kas_message_status_value {
    KAS_EINFO_UNKNOWN=-1,	// unknown status (should not happen)
    KAS_EINFO_NOSTATUS=0,	// status not set (not in INFO_DONE state)
    KAS_EINFO_ERROR=1,		// filter error
    KAS_EINFO_REJECT=2,		// message rejected for all recipients
    KAS_EINFO_BLACKHOLE=3,	// message blackholed for all recipients
    KAS_EINFO_BOUNCE=4,		// message bounced (see recipient list for 'bouncer' list)

  // several types of accept messages
    KAS_EINFO_ACCEPT_TRUSTED=5, // message accepted as Trusted (whitelisted)
    KAS_EINFO_ACCEPT_CLEAN=6,	// message is not spam
    KAS_EINFO_ACCEPT_PROBABLESPAM=7, // message should be marked as Probable spam
    KAS_EINFO_ACCEPT_SPAM=8,	// message should be marked as SPAM
    KAS_EINFO_ACCEPT_USERDEFINED=9, // unknown filter answer in
				    // X-SpamTest-Status header field
    KAS_EINFO_ACCEPT_FORMAL=10, // Formal message
    KAS_EINFO_ACCEPT_BLACKLISTED=11, // Blacklisted message
};


typedef struct {
    pid_t			   pid;   // scaner PID 
    enum kas_progress_cb_value     state; // see above for KAS_INFO_*
    enum kas_message_status_value  message_status; // KAS_EINFO, valid only for KAS_INFO_DONE

    // message info
    char	*rcpt; // recipient
    char *content_categories; // message content category list, separated by '|' char
    char *detection_methods;  // detection method list
    char *extended_info;      // extended scaner info (profile versions, matched rules and so)
}kas_progress_info_t;


// Progress callback prototype
typedef void (kas_cb_single_progress_t)(kas_session_t		* session,
				       kas_progress_info_t	*info,
                                       void			* param);

typedef void (kas_cb_multi_progress_t)(kas_multisession_t	* multisession,
				       unsigned int		index,
				       kas_progress_info_t	*info,
                                       void			* param);


typedef struct
{
    char *aname;
    char *avalue;
}kas_attribute_t;

// message metadata
typedef struct  
{
    char *relay_ip;  // sending relay IP in "1.2.3.4" string format
    char *helo_name; // HELO string
    char *mail_from; // MAIL FROM field without <> , empty string for empty sender
    int   nrcpts;    // Recipient count
    char **rcptlist; // Recipient array (e-mails without <>)
    int   nattrs;    // attribute count
    kas_attribute_t *attrlist; // attribute array
}kas_metadata_t;


// internal structures

typedef int (kas_single_io_cb_t)(kas_session_t *session);
typedef struct{
    size_t		alloc;
    size_t		used;
    unsigned char	*data;
} kas_buffer_t;


// kas_session_t data:
struct kas_session_s {
    // runtime data
    pid_t		pid;		// corresponding ap-mailfilter pid or 0
    int			socket;		// ap-mailfilter socket
    int			filter_rss;	// Filter process resident size

    // parameters set by CreateSession()
    unsigned int	max_spam_size;
    unsigned int	scanner_rss_limit;
    int			read_timeout;
    int			write_timeout;
    // set by Connect()
    int			connect_timeout;

    // Callbacks
    // status callback
    kas_cb_single_status_t	*status_cb;
    void			*status_cb_param;
    // progress cb
    kas_cb_single_progress_t    *progress_cb;
    void			*progress_cb_param;

    // input data
    // file descriptor (opened via ScanFile)
    int			datafile_fd; // opened datafile handle or -1 
    off_t		data_size;// 
				  // remaining data to send|total size for chunked
    char		*data_ptr; 

    // status
    kas_status_t	old_status;
    kas_status_t	current_status;

    
    // Internal data, do not touch

    off_t		file_pos;
    // metadata copy
    kas_metadata_t	*metadata;
    // last I/O performed
    struct timeval	last_io;
    struct timeval	scan_start;
    // io callback
    kas_single_io_cb_t  *write_func;    
    kas_single_io_cb_t  *read_func;    
    // internal structures
    kas_buffer_t	wr_buf;
    kas_buffer_t	rd_buf;
    kas_buffer_t	fi_buf;
    int			wasmalloc; //  data was allocated, so should be
				   // freeed at KasSSessionDestroy
    
    // filter internal state 
    int		        metadata_start;
    int			header_sent;
    int			line_complete;
    int			end_sent;
    int			standard_proto; // standard mode or looped mode
    int			chunked_write;
    // response state, see protocol desription
    char		*reason;
    int			xstatus;
    int			resp_count;
    // message status
    int			final_status;
    char		*content_categories;
    char		*detection_methods;
    char		*extended_info;
    // at last
    unsigned int	magick;
};

typedef struct {
    unsigned int		index;
    struct kas_multisession_s   *multisession;
    kas_cb_multi_status_t	*status_cb;
    void			*status_cb_data;
    kas_cb_multi_progress_t	*progress_cb;
    void			*progress_cb_data;
} kas_multicallback_gateway_t;

typedef struct {
    kas_session_t		kas_session;
    kas_multicallback_gateway_t cb_data;
}__kas_multisession_entry_t;

struct kas_multisession_s {
    unsigned int		count;
    struct pollfd		*pfds; // to use in ProcessEvents
    int				*socket_read;
    int				*socket_write;
    __kas_multisession_entry_t	sessions[1]; // really, count is allocated
};

#define KAS_MAGICK 0x189a45bc

EXTERN_C_END

#endif
