/* -*- C -*-
 * File: kas_client.h
 *
 * Created: Sat Oct  2 09:47:53 2004
 */
#ifndef __KAS_MCLIENT_H
#define __KAS_MCLIENT_H

#include "kas_defs.h"
#include <poll.h>
#include <sys/types.h>
#include <sys/time.h>

EXTERN_C_BEGIN

/* 
Creates new KAS session object

Parameters:
num_sessions	- number of sessions to be created
buf_size	- total I/O buffers size (8kb minimum) for each session
read_timeout	- timeout for elementary read() call in milliseconds
write_timeout	- timeout for  write() call in milliseconds
max_spam_size_in_bytes - larger messages will not be passed to scanner
scanner_data_limit_in_kbytes - Each scanner reports its memory usage 
		before entering IDLE state. If reported value is
		greater than scanner_data_limit, than status_callback
		will be called with event type KAS_STATUS_WANTRESTART
status_cb	- status callback
status_cb_data  - data to be passed to callback call

Return:
Pointer to allocated (or initialized) structure
NULL on error

Callbacks: none

*/
kas_multisession_t *
KasMCreateSession(unsigned int num_sessions,
		  unsigned int buf_size,
		  int read_timeout,
		  int write_timeout,
		  unsigned int max_spam_size_in_bytes,
		  unsigned int scanner_data_limit_in_kbytes,
		  kas_cb_multi_status_t *status_cb,
		  void *status_cb_data);

/*
Disconnects and destroys KAS session
Parameters:
sesptr - pointer to structure
Return: nothing
Callbacks: none
*/

void KasMSessionDestroy(kas_multisession_t *_multisession);


/*
KasMConnect - connect socket to scan daemon

Parameters:

sesptr		- pointer to multisession data
index		- index to peek scaner session
daemon_addr	- Daemon to connect to.
		Format:
		tcp:host:port - TCP connect
		unix:/path/to/socket - unix socket connect
		This value should be same as Listen parameter in
		ap-process-server.conf

timeout		- connect timeout in milliseconds.

Return:

KASerrPARAMS_NULLPTR	- null pointer passed in sesptr
KASerrPARAMS_NOTINITED	- session not created by Kas*CreateSession
KASerrALREADY_CONNECTED - session not in DISCONNECTED state
KASerrUNABLE_TO_CONNECT - connect() return error or incorrect daemon
			 address
KASerrINDEX_RANGE	- index out of range
KASerrIN_PROGRESS	- async. connect initiated
KASerrOK		- connected immediately

Callbacks:
this function will call status callback

*/
int  KasMConnect(kas_multisession_t *sesptr, unsigned int index,
		 const char *daemon_addr, int timeout);

/*
KasMPeekScanner - select scaner with status matches mask

Parameters:

sesptr		- pointer to multisession data
mask		- KAS_STATUS_* flag(s) to select scaner

Return:

-1	- cannot find scanner or errors
>=0	- scanner index to use

Callbacks: none

*/
int KasMPeekScanner(kas_multisession_t *sesptr,int mask);

/*
KasMCountScanners - count scanners in a given state

Parameters:

sesptr		- pointer to multisession data
mask		- KAS_STATUS_* flag(s)

Return:

-1	- cannot find scanner
>=0	- scanner count

Callbacks: none

*/
int KasMCountScanners(kas_multisession_t *sesptr,int mask);



/*
KasMDisconnect - disconnect session and go to DISCONNECTED state
Parameters:

sesptr		- pointer to session data

Return:
KASerrPARAMS_NULLPTR	- null pointer passed in sesptr
KASerrPARAMS_NOTINITED	- session not created by Kas*CreateSession
KASerrINDEX_RANGE	- index out of range
KASerrOK		- disconnected

Callbacks: none

*/
int  KasMDisconnect(kas_multisession_t *sesptr, unsigned int index );


/*
KasMWaitIdle - wait for IDLE status for some count of sessions

Parameters:
sesptr		- pointer to session data
count		- wanted count of idle scanners
timeout		- maximum time to wait to 

Return:
KASerrPARAMS_NULLPTR	- null pointer passed in sesptr
KASerrPARAMS_NOTINITED	- session not created by Kas*CreateSession
KASerrOK		- status reached
KASerrIN_PROGRESS	- timeout expired, but status not reached
KASerrINDEX_RANGE	- index out of range
KASerrDISCONNECTED	- session disconnected (before call or due to error)

Callbacks:
this function will call status callback
this function may call progress callback
*/

int  KasMWaitIdle(kas_multisession_t *sessp, int count, int timeout);


/*
KasMProcessEvents - perform one I/O cycle. This function is
near equivalent to 
KasMPreparePoll();
poll();
KasMProcessPollResults();
sequence

Parameters:
sesptr		- pointer to session data
timeout		- maximum time to wait to 

Return:
KASerrPARAMS_NULLPTR	- null pointer passed in sesptr
KASerrPARAMS_NOTINITED	- session not created by Kas*CreateSession
KASerrOK		- status reached
KASerrIN_PROGRESS	- timeout expired, but status not reached
KASerrDISCONNECTED	- session disconnected (before call or due to error)
KASerrINDEX_RANGE	- index out of range

Callbacks:
this function will call status callbacks
this function may call progress callbacks
*/

int KasMProcessEvents(kas_multisession_t *sessp, int timeout);


/* 
KasMPing - initiate ping to scan daemon for indicate client activity.
 ping should be completed by KasMWaitStatus or KasMProcessEvents

Parameters:
sesptr		- pointer to session data
index		- scanner index to use

Return:
KASerrPARAMS_NULLPTR	- null pointer passed in sesptr
KASerrPARAMS_NOTINITED	- session not created by Kas*CreateSession
KASerrIN_PROGRESS	- ping initiated
KASerrINDEX_RANGE	- index out of range
KASerrBUSY		- session busy for another activity

Callbacks: none

*/

int KasMPing(kas_multisession_t *s, unsigned int index);


/*
  KasMScanFile - submit file to antispam engine
Parameters:
sesptr		- pointer to session data
message_metadata - message metadata (relay IP, HELO, MAIL FROM and so
			on)
filename_to_scan - filename of file with message
progress_callback - to be called while message processing
callback_param	- to be passed to callback

Return:
KASerrPARAMS_NULLPTR	- null pointer passed in sesptr or pollfds
KASerrPARAMS_NOTINITED	- session not created by Kas*CreateSession
KASerrBUSY		- session not IDLE
KASerrFILEERROR		- cannot open()/stat() file
KASerrOK		- file 'scanned' immediately (file larger than
			max_spam_size_in_bytes, so CLEAN status
			returned
KASerrIN_PROGRESS	- file submitted

Callbacks:
this function may call progress callback
*/			

int KasMScanFile(kas_multisession_t *s,
		 unsigned int index_to_use,
		 kas_metadata_t * message_metadata,
		 char *filename_to_scan,
		 kas_cb_multi_progress_t *progress_callback,
		 void *callback_param);

/*
  KasMScanOpenFile - submit open file to antispam engine
Parameters:
sesptr		- pointer to session data
message_metadata - message metadata (relay IP, HELO, MAIL FROM and so
			on)
file_fd		- file descriptor (lseek()-ed to message start)
file_data_size  - length of message (in bytes)
filename_to_scan - filename of file with message
progress_callback - to be called while message processing
callback_param	- to be passed to callback

Return:
KASerrPARAMS_NULLPTR	- null pointer passed in sesptr or pollfds
KASerrPARAMS_NOTINITED	- session not created by Kas*CreateSession
KASerrBUSY		- session not IDLE
KASerrOK		- file 'scanned' immediately (file larger than
			max_spam_size_in_bytes, so CLEAN status
			returned
KASerrIN_PROGRESS	- file submitted

Callbacks:
this function may call progress callback

This function will restore file postion after scan completion.
This function will not close passed descriptor.
This function will change file position during scan process.

*/			
int KasMScanOpenFile(kas_multisession_t *s,
		     unsigned int index_to_use,
		     kas_metadata_t * message_metadata,
		     int  file_fd,
		     off_t file_data_size,
		     kas_cb_multi_progress_t *progress_callback,
		     void *callback_param);



/* 
KasMPreparePoll - prepare poll structure for later poll() call

Parameters:
sesptr		- pointer to session data
pollfds		- array of struct pollfd
max_index	- size of pollfds list

Return:
KASerrPARAMS_NULLPTR	- null pointer passed in sesptr or pollfds
KASerrPARAMS_NOTINITED	- session not created by Kas*CreateSession
KASerrDISCONNECTED	- session not connected
KASerrOK		- pollfd fields prepared

Callbacks: none

*/

int  KasMPreparePoll(kas_multisession_t *sesptr,struct pollfd *pollfds,unsigned int max_index);


/*
  KasMProcessPollResult - do some I/O (and other processing) after poll

Parameters:
sesptr		- pointer to session data
pollfds		- array of struct pollfd
max_index	- size of pollfds list

Return:
KASerrPARAMS_NULLPTR	- null pointer passed in sesptr or pollfds
KASerrPARAMS_NOTINITED	- session not created by KasS*reateSession
KASerrOK		- current processing finished, IDLE state
KASerrIN_PROGRESS	- some I/O done, but current operation still
			in progress
KASerrREAD_ERROR	- read() returns an error, session disconnected
KASerrWRITE_ERROR	- write() returns an error, session disconnected
KASerrUNABLE_TO_CONNECT - cannot connect (if current operation is
			async. connect)

Callbacks:
this function will call status callback
this function may call progress callback

*/
int  KasMProcessPollResult(kas_multisession_t *sesptr, struct pollfd* pollfds, unsigned int max_index);

/* 
KasMPrepareSelect - prepare poll structure for later poll() call

Parameters:
sesptr		- pointer to session data
readfd		- fd_set to use as 2nd parameter for select()
writefd		- fd_set to use as 3rd parameter for select()

Return:
KASerrPARAMS_NULLPTR	- null pointer passed in sesptr or fd-sets
KASerrPARAMS_NOTINITED	- session not created by KasSCreateSession
KASerrDISCONNECTED	- session not connected
KASerrOK		- pollfd fields prepared

Callbacks: none

*/

int KasMPrepareSelect(kas_multisession_t *sesptr,fd_set *readfd, fd_set *writefd);

/*
  KasMProcessSelectResult - do some I/O (and other processing) after select()

Parameters:
sesptr		- pointer to session data
readfd		- fd_set used  as 2nd parameter in select()
writefd		- fd_set used 3rd parameter in select()

Return:
KASerrPARAMS_NULLPTR	- null pointer passed in sesptr or fd_set-s
KASerrPARAMS_NOTINITED	- session not created by KasSCreateSession
KASerrOK		- current processing finished, IDLE state
KASerrIN_PROGRESS	- some I/O done, but current operation still
			in progress
KASerrREAD_ERROR	- read() returns an error, session disconnected
KASerrWRITE_ERROR	- write() returns an error, session disconnected
KASerrUNABLE_TO_CONNECT - cannot connect (if current operation is
			async. connect)

Callbacks:
this function will call status callback
this function may call progress callback

*/
int  KasMProcessSelectResult(kas_multisession_t *sesptr, fd_set *readfd, fd_set *writefd);

EXTERN_C_END


#endif
